@@ -13,6 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from six.moves.urllib import parse as urlparse
+
 from saharaclient.api import base
 
 
@@ -24,11 +26,13 @@ class JobBinaryInternalsManager(base.ResourceManager):
     resource_class = JobBinaryInternal
 
     def create(self, name, data):
-        return self._update('/job-binary-internals/%s' % name, data,
+        return self._update('/job-binary-internals/%s' %
+                            urlparse.quote(name.encode('utf-8')), data,
                             'job_binary_internal', dump_json=False)
 
-    def list(self):
-        return self._list('/job-binary-internals', 'binaries')
+    def list(self, search_opts=None):
+        query = base.get_query_string(search_opts)
+        return self._list('/job-binary-internals%s' % query, 'binaries')
 
     def get(self, job_binary_id):
         return self._get('/job-binary-internals/%s' % job_binary_id,
@@ -36,3 +40,12 @@ class JobBinaryInternalsManager(base.ResourceManager):
 
     def delete(self, job_binary_id):
         self._delete('/job-binary-internals/%s' % job_binary_id)
+
+    def update(self, job_binary_id, name=None, is_public=None,
+               is_protected=None):
+
+        data = {}
+        self._copy_if_defined(data, name=name, is_public=is_public,
+                              is_protected=is_protected)
+
+        return self._patch('/job-binary-internals/%s' % job_binary_id, data)
